#!/usr/bin/env python
# -*- coding: utf-8 -*-

#***************************************************************************
#*   neposidekick.py - several tools to manage nepomuk user resources      *
#*                                                                         *
#*   Copyright                                                             *
#*   (C) 2011 Ignacio Serantes <kde@aynoa.net>                             *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU General Public License as published by  *
#*   the Free Software Foundation; either version 2 of the License, or     *
#*   (at your option) any later version.                                   *
#*                                                                         *
#*   This program is distributed in the hope that it will be useful,       *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU General Public License for more details.                          *
#*                                                                         *
#*   You should have received a copy of the GNU General Public License     *
#*   along with this program; if not, write to the                         *
#*   Free Software Foundation, Inc.,                                       *
#*   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
#***************************************************************************

# changelog
#
# v0.2 - 2011/02/10
# - program has been totally rewriten
# - adding tag to symlinks works even if symlink is pointing nowhere
# - edit resources icon changed to accessories-text-editor
# - new text resource formats 'ntr' y 'nhr' with support to all nepumuk resource properties
# - all v0.1 formats are now deprecated but still supported to load
# - many changes in parameters so check the help
# - new commands: addtags, cleandir, cleanfile, removetags
# - some commands works even if file does not exists, for example cleantags
# - parameter --human-readable creates text resouces with an easy to understand, but limited, format
# - service menu save uses --human-readable format
# - --recursive option added
# - new parameter --ontology that supports short version (nao#hasTag) or long version (http://www.semanticdesktop.org/ontologies/2007/08/15/nao#hasTag) names
# - when nepomuk resource file is saved a backup is created
#
# v0.1 - 2011/01/10
# - first release

# begin_1 neposidekick.py
import codecs, csv, os, sys
from datetime import datetime

from PyQt4 import QtCore
from PyKDE4 import kdecore
from PyKDE4.nepomuk import *
from PyKDE4.soprano import *
# end_1 neposidekick.py
# begin nsglobals.py
PROGRAM_NAME = os.path.basename(sys.argv[0])
PROGRAM_BASENAME = os.path.splitext(PROGRAM_NAME)[0]
PROGRAM_PATH = os.path.dirname(os.path.abspath(sys.argv[0]))
PROGRAM_VERSION_VERSION = 'v0.2'
PROGRAM_VERSION_DATE = '2011/02/10'
#PROGRAM_VERSION_DATE = '2011/01/10'
PROGRAM_AUTHOR_NAME = 'Ig'
PROGRAM_AUTHOR_EMAIL = 'kde@aynoa.net'

KDE_LOCAL_PREFIX = os.popen('kde4-config --path data').read().strip()
KDE_LOCAL_PREFIX = KDE_LOCAL_PREFIX[0:KDE_LOCAL_PREFIX.find(':')]
DEFAULT_RESOURCE_FILE_NAME = '.' + PROGRAM_BASENAME + '_resource'
DEFAULT_RESOURCE_FILE = KDE_LOCAL_PREFIX + 'nepomuk/' + PROGRAM_BASENAME + '_resource'

RECORD_ID_ALL = 'a' # Deprecated.
RECORD_ID_FILE = 'nhr'
RECORD_ID_COMMENT = 'c' # Deprecated.
RECORD_ID_DIRECTORY = 'd' # Deprecated.
RECORD_ID_PROPERTY = 'p' # Deprecated.
RECORD_ID_RATING = 'r' # Deprecated.
RECORD_ID_RESOURCE = 'ntr'
RECORD_ID_TAG = 't' # Deprecated.
RECORD_ID_EXPORT_IMPORT = 'e' # Deprecated.

RDF_SCHEMA_RESOURCE = 'http://www.w3.org/2000/01/rdf-schema#Resource'

gSERVICE_MENU_MODE = False
gRECURSIVE_MODE = False
gSysEncoding = 'utf-8' # Change this for a detection system.
gWorkDirectory = ''

knownOntologies = [ \
                    ['nao', '2007/08/15'], ['ncal', '2007/04/02'], \
                    ['nco', '2007/03/22'], ['nexif', '2007/05/10'], \
                    ['nfo', '2007/03/22'], ['nid3', '2007/05/10'], \
                    ['nie', '2007/01/19'], ['nmm', '2009/02/19'], \
                    ['nmo', '2007/03/22'], ['nrl', '2007/08/15'], \
                    ['pimo', '2007/11/01'], ['tmo',  '2008/05/20'] \
                ]


def setGlobalValue(name, value = None):
    exec 'global %(name)s\n%(name)s = %(value)s' % {'name': name, 'value': value}


def getGlobalValue(name):
    try:
        return eval(name)
    except:
        return None


def strTruncToPrint(str, size):
    if size < len(str):
        shortString = ' ... '
        if size < len(shortString):
            return str

        size = size - len(shortString)
        pos = size / 2
        return str[0:pos] + shortString + str[pos - size:]

    else:
        return str


def customPrint(str, mode = 'passivepopup'):
    global gSERVICE_MENU_MODE, PROGRAM_NAME
    print str.encode('utf-8')
    if not gSERVICE_MENU_MODE or mode == 'stdout':
        pass
    else:
        cmd = 'kdialog --title "%(1)s" --%(2)s "%(3)s"' % {'1': PROGRAM_NAME, '2': mode, '3': str}
        os.system(toUtf8(cmd))


def DEBUG(str):
    customPrint('DEBUG: ' + str, 'stdout')


def timestamp():
    return datetime.now().isoformat()[0:23] + 'Z'


def addBS(path = '.'):
    path = path.strip()
    if path == '':
        return ''

    else:
        if path[-1] == '/':
            return path
        else:
            return path + '/'


def toUtf8(string):
    try:
        #if vartype(string) == 'unicode':
        if vartype(string) == 'QString':
            return string.toUtf8() # REVISAR: Esto no es coherente con el resto.

        else:
            return string.encode(gSysEncoding)

        #else:
        #    return string

    except:
        return string


def toUnicode(string):
    try:

        if vartype(string) == 'QString':
            return unicode(str(string.toUtf8()), gSysEncoding)

        if vartype(string) == 'unicode':
            return string

        return unicode(string, gSysEncoding)

    except:
        return string


def iif(condition = True, value = '', optionalValue = ''):
    return value if condition else optionalValue


def lindex(items, value, column = None):
    try:
        if vartype(items[0]) != 'list':
            column = None

        if column == None:
            result = next((i for i, element in enumerate(items) if value in element), None)

        else:
            result = next((i for i, element in enumerate(items) if value == element[column]), None)

    except:
        result = None

    return result


def lvalue(items, value, searchColumn = 0, valueColumn = 0):
    try:
        value = items[lindex(items, value, searchColumn)][valueColumn]

    except:
        value = None

    return value


def vartype(var):
    return type(var).__name__


def NEO(name = ''):
    ontology, property = name.strip().split('#')
    date = lvalue(knownOntologies, ontology, 0, 1)
    if date != None:
        value = 'http://www.semanticdesktop.org/ontologies/' + date + '/' + ontology + '#' + property

    else:
        value = 'Soprano.Vocabulary.%(1)s.%(2)s().toString()' % {'1': ontology.upper(), '2': property}
        try:
            value = eval(value)

        except:
            value = ''

    return value

    #if ontology == 'nao':
        #date = '2007/08/15'
    #elif ontology == 'ncal':
        #date = '2007/04/02'
    #elif ontology == 'nco':
        #date = '2007/03/22'
    #elif ontology == 'nexif':
        #date = '2007/05/10'
    #elif ontology == 'nfo':
        #date = '2007/03/22'
    #elif ontology == 'nid3':
        #date = '2007/05/10'
    #elif ontology == 'nie':
        #date = '2007/01/19'
    #elif ontology == 'nmm':
        #date = '2009/02/19'
    #elif ontology == 'nmo':
        #date = '2007/03/22'
    #elif ontology == 'nrl':
        #date = '2007/08/15'
    #elif ontology == 'pimo':
        #date = '2007/11/01'
    #elif ontology == 'tmo':
        #date = '2008/05/20'


def findResourceFiles(rootPath = './'):
    name = DEFAULT_RESOURCE_FILE_NAME
    resources = []
    for dirPath, dirName, fileNames in os.walk(unicode(rootPath)):
        for fileName in fileNames:
            if fileName == name:
                resources += [addBS(dirPath) + fileName]

    return len(resources), sorted(resources)


def findRecurseDirectories(rootPath = './', hidden = False):
    resources = []
    rootPath = os.path.normpath(rootPath)
    for dirPath, dirName, fileNames in os.walk(unicode(rootPath)):
        if not hidden and os.path.basename(dirPath) == '.':
            continue

        resources += [addBS(dirPath)]

    return len(resources), sorted(resources)

# end nsglobals.py# begin nsresource.py
class nsResource(Nepomuk.Resource):

    appendMode = False
    exportOnlyFilesWithTRC = True
    exportTimestamps = False
    nsp = None

    def __init__(self, par1 = None, par2 = None, par3 = None, par4 = None):

        if par1 != None and par1.__class__.__name__ == 'nsProgram':
            self.nsp = par1
            par1 = par2
            par2 = par3
            par3 = par4

        if vartype(par1) == 'list':
            type, id = self.identifyType(par1, par2)

            if self.nsp != None:
                if not type in self.nsp.ontologies:
                    return None

            if id == '':
                Nepomuk.Resource.__init__(self)

            else:
                if type == NEO('nfo#FileDataObject'):
                    if par2 == None:
                        fileInfo = QtCore.QFileInfo(id)

                    else:
                        fileInfo = QtCore.QFileInfo(addBS(par2) + id)

                    if not (fileInfo.isSymLink() or fileInfo.exists()):
                        return None

                    id = kdecore.KUrl(fileInfo.absoluteFilePath())
                    Nepomuk.Resource.__init__(self, id, kdecore.KUrl(type))

                elif type == NEO('nao#Tag'):
                    #tag = Nepomuk.Tag(id)
                    #tag.setLabel(id)
                    #Nepomuk.Resource.__init__(self, tag.resourceUri())
                    Nepomuk.Resource.__init__(self, id, kdecore.KUrl(type))

                else:
                    Nepomuk.Resource.__init__(self)

            # par1 -- list with resources.
            # par2 -- work directory.
            if par2 == None:
                self.fromText(par1)

            else:
                self.fromText(par1, par2)

        else:

            if par3 != None:
                Nepomuk.Resource.__init__(self, par1, par2, par3)

            elif par2 != None:
                Nepomuk.Resource.__init__(self, par1, par2)

            elif par1 != None:
                Nepomuk.Resource.__init__(self, par1)

            else:
                Nepomuk.Resource.__init__(self)

            #self.ID = self.label()


    def __del__(self):
        pass


    _private_ID = None

    def _get_ID(self):
        result = self._private_ID
        if result != None and result != '':
            return result

        result = self.label()
        if result != '':
            return result

        result = self.property(NEO('nie#url')).toString()
        if result != '':
            result = toUnicode(result)
            return os.path.basename(result)

        return None

    def _set_ID(self, value):
        #if vartype(value) == 'QString':
        #    self._private_ID = QtCore.QFileInfo(value).fileName()

        #else:
        #    self._private_ID = os.path.basename(toUnicode(value))
        self._private_ID = value

    ID = property(_get_ID, _set_ID)


    def getOntologyLabel(self, resource = None):

        if resource == None:
            resource = self

        value = ''
        if resource.type() == NEO('nco#Contact') or resource.type == NEO('nco#PersonContact'):
            value = resource.property(NEO('nco#fullname')).toString()

        elif resource.type() == NEO('nmm#MusicAlbum') or resource.type == NEO('nmm#musicAlbum'):
            value = resource.property(NEO('nie#title')).toString()

        elif resource.type() == RDF_SCHEMA_RESOURCE:
            value = resource.property(NEO('nie#title')).toString()
            #if value == '':
            #    pdb.set_trace()

        #else:
            #pass

        if value == '':
            value = resource.property(NEO('nie#url')).toString()
            if value == '':
                value = resource.property(NEO('nie#prefLabel')).toString()
                if value == '':
                    if resource.label() != '':
                        value = resource.label()
                    else:
                        value = resource.uri()

        return value


    def toText(self, mode = 's'):
        if mode.find('s') >= 0:
            shorted = True

        else:
            shorted = False

        if mode.find('h') >= 0:
            # Human format.
            # nhr,|<file>|,|<comment>|,|<rating>|[,|tag|]...
            fileName = QtCore.QFileInfo(self.property(NEO('nie#url')).toString().trimmed()).fileName()
            #if self.ID == '':
            #    self.ID = fileName

            text = ''
            if fileName != '':
                text += RECORD_ID_FILE + ',|%s|' % fileName
                text += ',|%s|' % self.description()
                if self.rating() == 0:
                    text += ',||'

                else:
                    text += ',|%s|' % self.rating()

                for tag in self.tags():
                    text += ',|%s|' % nsResource(tag).property(NEO('nao#prefLabel')).toString().trimmed()

            #else:
            #    if not shorted:
            #        text = RECORD_ID_FILE + ',|%s|,||,||' % fileName

        elif mode.find('l') > 0:
            # List format.
            fileName = self.property(NEO('nie#url')).toString().trimmed()
            text = ''
            if fileName != '':
                if fileName[0:7] == 'file://':
                    fileName = fileName[7:]
                fileName = QtCore.QFileInfo(fileName)
                text += "---------"
                text += "\nName:     %s" % fileName.fileName()
                text += "\nLocation: %s" % fileName.absolutePath()
                if self.rating() > 0:
                    text += '\nRating:   %s' % self.rating()
                allTags = ''
                for tag in self.tags():
                    if allTags != '':
                        allTags += ', '
                    allTags += '%s' % nsResource(tag).property(NEO('nao#prefLabel')).toString().trimmed()

                if allTags != '':
                    text += '\nTags:     %s' % allTags

                if self.description() != '':
                    text += '\nComment:\n%s' % self.description()


        else:
            # Full format.

            text = ''

            # Main type
            text += ',|%s|' % toUnicode(self.type())
            # The other types.
            for type in self.types():
                type = type.toString()
                if type != self.type():
                    if type[0:13] == 'nepomuk:/res/':
                        resource = nsResource(type)
                        text += ",|%s|" % toUnicode(resource.getOntologyLabel())

                    else:
                        text += ",|%s|" % toUnicode(type)

            text += ','

            thereIsTRC = False
            data = []
            for propertyUrl in self.properties():
                value = self.property(propertyUrl).toString().trimmed()
                strPropertyUrl = propertyUrl.toString()
                if not self.exportTimestamps and strPropertyUrl == NEO('nao#created'):
                    continue

                elif strPropertyUrl == NEO('nao#description'):
                    if value != '':
                        thereIsTRC = True

                elif strPropertyUrl == NEO('nao#hasTag'):
                    if value != '':
                        thereIsTRC = True

                elif strPropertyUrl == NEO('nao#identifier'):
                    pass

                elif not self.exportTimestamps and (strPropertyUrl == NEO('nie#lastModified') or strPropertyUrl == NEO('nao#lastModified')):
                    continue

                elif strPropertyUrl == NEO('nao#numericRating'):
                    if value == '0':
                        # los rating de 0 no se incluyen si la versión es la corta.
                        value = ''

                    else:
                        thereIsTRC = True

                elif strPropertyUrl == NEO('nie#url'):
                    pass

                #else:
                    #pass

                if not (shorted and value == ''):
                    propertyName = propertyUrl.toString()
                    data.append([toUnicode(propertyName), toUnicode(value)])

            if not self.type() == NEO('nfo#FileDataObject') or not self.exportOnlyFilesWithTRC or thereIsTRC:
                data = sorted(data)
                for item in data:
                    text += ",|%s|" % item[0]

                    if item[1][0:13] == 'nepomuk:/res/':
                        resources = item[1].split(',')
                        for item in resources:
                            resource = nsResource(item)
                            value = resource.getOntologyLabel()
                            text += ",|%(value)s|,|%(type)s|" % {'value': toUnicode(value), 'type': toUnicode(resource.type())}

                    else:
                        text += ",|%s|" % item[1]

                    text += ','

                if shorted and (text == ',||,' or text == ',|' + RDF_SCHEMA_RESOURCE + '|,'):
                    text = ''

                else:
                    if mode.find('f') >= 0:
                        text = ',|%s|,' % timestamp() + text
                        text = ',|%s|' % QtCore.QFileInfo(self.property(NEO('nie#url')).toString().trimmed()).fileName() + text

                    else:
                        text = ',' + text

                    text = RECORD_ID_RESOURCE + text # Record id.

            else:
                text = ''

        return text


    def addPropertyFromText(self, property, value, type):
        if property == '':
            return

        pValue = 'p'
        if property == NEO('nao#created'):
            pValue += '_'

        elif property == NEO('nao#identifier'):
            pValue += '*'

        elif property == NEO('nao#lastModified'):
            pValue += '_'

        elif property == NEO('nao#prefLabel'):
            pValue += '!'

        elif property == NEO('nie#url'):
            pValue += '*'

        else:
            pValue += '+'

        #if self.nsp != None:
            #self.nsp.DEBUG('>>> %(p)s: %(property)s v: %(value)s t: %(type)s' \
                    #% {'p': pValue, \
                        #'property': strTruncToPrint(os.path.basename(property).ljust(20), 20), \
                        #'value': strTruncToPrint(value.ljust(30), 30), \
                        #'type': os.path.basename(type)})

        if pValue != 'p_': # Don't import certain properties.
            if type == '':
                # A property.
                if property == 'nao:altLabel':
                    self.addAltLabel(value)

                elif property == NEO('nao#description'):
                    self.setDescription(value)
                    if self.nsp != None:
                        self.nsp.DEBUG('self.setDescription(%s)' % value)

                elif property == NEO('nao#numericRating'):
                    if value == '':
                        value = 0
                    if value > 0:
                        self.setRating(int(value))
                        if self.nsp != None:
                            self.nsp.DEBUG('self.setRating(%s)' % int(value))

                elif property == 'nao:nao:hasSymbol':
                    self.addSymbol(value)

                elif property == 'nao:prefLabel':
                    self.setLabel(value)

                elif property == NEO('nie#url'):
                    pass

                else:
                    if self.hasProperty(property):
                        self.setProperty(QtCore.QUrl(property), Nepomuk.Variant(value))
                        if self.nsp != None:
                            self.nsp.DEBUG('self.setProperty(%(p)s, %(v)s)' % {'p': os.path.basename(property), 'v': value})

                    else:
                        self.addProperty(QtCore.QUrl(property), Nepomuk.Variant(value))
                        if self.nsp != None:
                            self.nsp.DEBUG('self.addProperty(%(p)s, %(v)s)' % {'p': os.path.basename(property), 'v': value})

            else:
                # A relation.
                #if property == NEO('nao#hasTag'):
                    #tag = Nepomuk.Tag(value)
                    #tag.setLabel(value)
                    #self.addTag(tag)
                    #if self.nsp != None:
                        #self.nsp.DEBUG('self.addTag(%s)' % value)

                #elif property == NEO('nao#isRelated'):
                    #resource = Nepomuk.Resource(kdecore.KUrl(value), kdecore.KUrl(type))
                    #self.addIsRelated(resource)
                    #if self.nsp != None:
                        #self.nsp.DEBUG('self.addIsRelated(%s)' % value)

                #elif property == NEO('nao#isTopicOf'):
                    #resource = Nepomuk.Resource(kdecore.KUrl(value), kdecore.KUrl(type))
                    #self.addIsTopicOff(resource)
                    #if self.nsp != None:
                        #self.nsp.DEBUG('self.addIsTopicOff(%s)' % value)

                #else:
                resource = Nepomuk.Resource(value, kdecore.KUrl(type))
                self.addProperty(QtCore.QUrl(property), Nepomuk.Variant(resource.resourceUri()))
                if self.nsp != None:
                    self.nsp.DEBUG("Relation '%s' added." % property)


    def convertFromHuman(self, textResource, workDir):
        if not (textResource[0] == RECORD_ID_FILE or textResource[0] == RECORD_ID_ALL):
            return ['']

        converted = []
        try:
            fileName = toUnicode(textResource[1])
            converted += [RECORD_ID_RESOURCE, fileName, '']
            converted += [NEO('nfo#FileDataObject'), RDF_SCHEMA_RESOURCE, u'']
            converted += [NEO('nie#url'), 'file://' + addBS(workDir) + fileName, u'']
            try:
                comment = toUnicode(textResource[2])
                converted += [NEO('nao#description'), comment, u'']
            except:
                pass

            try:
                rating = toUnicode(textResource[3])
                converted += [NEO('nao#numericRating'), rating, u'']
            except:
                pass

            for i in range(4, len(textResource)):
                converted += [NEO('nao#hasTag'), toUnicode(textResource[i]), NEO('nao#Tag'), '']

        except:
            pass

        return converted


    def identifyType(self, textResource, workDir):
        # El formato humano es fácil.
        if textResource[0] == RECORD_ID_FILE or textResource[0] == RECORD_ID_ALL:
            if self.nsp != None:
                self.nsp.DEBUG("identifyType: '%(ontology)s', '%(id)s'" % {'ontology': 'nfo#FileDataObject', 'id': textResource[1]})
            #self.ID = toUnicode(textResource[1])
            return NEO('nfo#FileDataObject'), toUnicode(textResource[1])

        # El formato completo necesita más trabajo.
        value = toUnicode(textResource[1].strip())
        nextColumnToProcess = 2
        if value == '':
            # Skip empty columns.
            for i in range(nextColumnToProcess, len(textResource)):
                if textResource[i] != '':
                    nextColumnToProcess = i
                    break

        else:
            # Skip not empty columns.
            for i in range(nextColumnToProcess, len(textResource)):
                if textResource[i] == '':
                    nextColumnToProcess = i
                    break

            # Skip empty columns.
            for i in range(nextColumnToProcess, len(textResource)):
                if textResource[i] != '':
                    nextColumnToProcess = i
                    break

        type = toUnicode(textResource[nextColumnToProcess].strip())

        if type == NEO('nfo#FileDataObject'):
            # El ID sale de nie#url
            i = lindex(textResource, NEO('nie#url'))
            if i >= 0:
                if workDir == None or workDir == '':
                    id = toUnicode(textResource[i+1])
                    if id[0:7] == 'file://':
                        id = id[7:]

                else:
                    id = toUnicode(os.path.basename(toUnicode(textResource[i+1])))

            else:
                id = ''

            #self.ID = id

        elif type == NEO('nao#Tag'):
            # El ID sale de nao#prefLabel
            #i = lindex(textResource, NEO('nao#prefLabel'))
            i = lindex(textResource, NEO('nao#identifier'))
            if i >= 0:
                id = toUnicode(textResource[i+1])

            else:
                id = ''

        else:
            id = ''

        return type, id


    def fromText(self, textResource, workDir = '', ontologies = []):
        if workDir != '' and workDir != None:
            if not os.path.isdir(workDir):
                if self.nsp != None:
                    self.nsp.pPrint("ERROR: '%s' don't exists." % workDir, 'error')
                return False

            workDir = addBS(workDir)

        else:
            workDir = ''

        if textResource[0] == RECORD_ID_FILE or textResource[0] == RECORD_ID_ALL:
            textResource = self.convertFromHuman(textResource, workDir)

        if textResource[0] != RECORD_ID_RESOURCE:
            if self.nsp != None:
                self.nsp.pPrint("ERROR: '%s' is an unknown data format." % textResource[0])
            return False

        # is a nfo#FileDataObject resource.
        # id, fileName[, timestamp],||,mainNepomukType[,nepomukType]...,||,...
        fileName =  toUnicode(textResource[1].strip())
        if fileName != '':
            # NAO(nfo#FileDataObject).
            if self.nsp != None:
                self.nsp.DEBUG('Filename : ' + fileName)
            timestamp = toUnicode(textResource[2].strip())
            if self.nsp != None:
                self.nsp.DEBUG('Timestamp: ' + timestamp)
            # timestamp could be available or not.
            if timestamp == '':
                nextColumnToProcess = 3

            else:
                nextColumnToProcess = 4

        else:
            # Resto.
            nextColumnToProcess = 2

        # Skip empty columns.
        for i in range(nextColumnToProcess, len(textResource)):
            if textResource[i] != '':
                nextColumnToProcess = i
                break

        # Get all nepomukTypes.
        nepomukTypes = [toUnicode(textResource[nextColumnToProcess])]
        nextColumnToProcess += 1

        for i in range(nextColumnToProcess, len(textResource)):
            value = toUnicode(textResource[i].strip())
            if value == '':
                nextColumnToProcess = i
                break

            else:
                nepomukTypes.append(value)

        # Skip empty columns.
        for i in range(nextColumnToProcess, len(textResource)):
            if textResource[i] != '':
                nextColumnToProcess = i
                break

        # Set resource types.
        types = []
        if self.nsp != None:
            dummyStr = ''
        for type in nepomukTypes:
            if self.nsp != None:
                if dummyStr == '':
                    dummyStr += os.path.basename(type)

                else:
                    dummyStr += ', ' + os.path.basename(type)

            types.append(QtCore.QUrl(type))

        self.setTypes(types)
        if self.nsp != None:
            self.nsp.DEBUG('nepomukTypes: %s' % dummyStr)

        # Property initialization.
        self.setAltLabels([])
        self.setDescription(value)
        self.setIsRelateds([])
        if self.rating() > 0:
            self.setRating(0)
        self.setSymbols([])

        curProperty = ''
        curValue = ''
        curType = ''
        for i in range(nextColumnToProcess, len(textResource)):
            value = toUnicode(textResource[i].strip())
            if value == '':
                self.addPropertyFromText(curProperty, curValue, curType)
                curProperty = curValue = curType = ''

            else:
                if curProperty == '':
                    curProperty = value

                elif curValue == '':
                    curValue = value

                else:
                    if curType == '':
                        curType = value
                    else:
                        self.addPropertyFromText(curProperty, curValue, curType)
                        curValue = value
                        curType = ''

        return True

# end nsresource.py
# begin nsprogram.py
class nsProgram():

    DEBUG_ENABLED = False

    backupExtension = '~'
    backupFiles = True
    command = ''
    enabled = False
    defaultOntologies = [NEO('nao#Tag'), NEO('nfo#FileDataObject')]
    outputFormat = 'full' # full, human, list
    lastCommand = ''
    localResourcesFileName = ''
    #ontologies = [NEO('nco#Contact'), NEO('nco#Gender'), NEO('nao#Tag'), NEO('nfo#FileDataObject'), NEO('nmm#MusicAlbum'), NEO('nco#OrganizationContact'), NEO('nco#PersonContact'), NEO('nfo#Website'), NEO('pimo#Country')]
    ontologies = defaultOntologies
    recursiveMode = False
    resourcesFileName = ''
    serviceMenuMode = False
    tags = []
    url = ''


    def __init__(self, parameters = None, run = True):

        if Nepomuk.ResourceManager.instance().init() != 0:
            self.pPrint("ERROR: Nepomuk not running.", 'error')
            self.enabled = False

        else:
            self.enabled = True

        self.localResourcesFileName = DEFAULT_RESOURCE_FILE_NAME
        self.resourcesFileName = DEFAULT_RESOURCE_FILE

        if parameters != None:
            self.parseParameters(parameters)
            if run:
                self.runCommand()


    def __del__(self):
        pass


    def pPrint(self, str = '', mode = 'passivepopup'):
        global PROGRAM_NAME
        print toUtf8(str)
        if not self.serviceMenuMode or mode == 'stdout':
            pass

        else:
            if str[0:3] == "#> ":
                str = str[3:]

            cmd = 'kdialog --title "%(1)s" --%(2)s "%(3)s"' % {'1': PROGRAM_NAME, '2': mode, '3': str}
            os.system(toUtf8(cmd))


    def DEBUG(self, str):
        if self.DEBUG_ENABLED:
            self.pPrint('DEBUG: ' + str, 'stdout')


    def getRootDir(self, url = None):
        if url == None:
            url = self.url

        if os.path.isdir(url):
            rootDir = url

        else:
            if not os.path.isfile(url):
                self.pPrint("ERROR: '%s' does not exists." % url, 'error')
                return None

            rootDir = toUnicode(os.path.dirname(url))

        if not os.path.isdir(rootDir):
            self.pPrint("ERROR: directory '%s' does not exists." % rootDir, 'error')
            return None

        return rootDir


    def allResourcesOfType(self, type):
        if not self.enabled:
            self.DEBUG('Nepomuk is not running!')
            return -1, None

        if type == '':
            self.pPrint("ERROR: bad parameter.", 'error')
            return -1, None

        nepomukType = Nepomuk.Types.Class(QtCore.QUrl(type))
        term = Nepomuk.Query.ResourceTypeTerm(nepomukType)
        query = Nepomuk.Query.Query(term)
        sparql = query.toSparqlQuery()
        self.DEBUG(sparql)

        model = Nepomuk.ResourceManager.instance().mainModel()
        data = model.executeQuery(sparql, Soprano.Query.QueryLanguageSparql)

        rows = []
        while data.next():
            bindingSet = data.currentBindings()
            for i in range(len(data.bindingNames())) :
                v = bindingSet.value(data.bindingNames()[i])
                #self.DEBUG(v.uri().toString())
                uri = v.uri()
                resource = nsResource(self, uri)
                if resource.type() == type:
                    rows.append(resource)
                #self.DEBUG("%(label)s, %(uri)s" % {'uri': resource.uri(), 'label': resource.label()})

        data.close()

        return len(rows), rows


    def commandAddTags(self, url = None, tags = []):
        if url == None:
            url = self.url

        if url == None or url == '':
            return False

        fileInfo = QtCore.QFileInfo(url)
        if not (fileInfo.isSymLink() or fileInfo.exists()):
            return False

        if tags == []:
            tags = self.tags

        if tags == []:
            return True

        try:
            oResource = nsResource(kdecore.KUrl(fileInfo.absoluteFilePath()))
            strTags = ''
            displayName = 'tag'
            for tag in tags:
                oTag = Nepomuk.Tag(tag)
                oTag.setLabel(tag)
                oResource.addTag(oTag)
                if strTags != '':
                    if tag == tags[len(tags) - 1]:
                        strTags += ' and '
                        displayName += 's'

                    else:
                        strTags += ', '

                strTags += "'%s'" % tag

            self.pPrint("#> %(tags)s %(displayname)s added to '%(url)s'." % {'tags': strTags, 'displayname': displayName, 'url': url})

        except:
            self.pPrint("#> Failed to add tags to '%s'." %  url)

        return True


    #def commandCleanAll(self):
        #self.commandHelp()

        #return True


    def commandCleanDir(self, url = None):
        if url == None:
            url = self.url

        rootDir = self.getRootDir(url)
        if rootDir == None:
            return False

        if self.recursiveMode:
            directoriesCount, directories = findRecurseDirectories(rootDir)

        else:
            directoriesCount = 1
            directories = [addBS(rootDir)]

        if self.recursiveMode:
            self.pPrint("#> Clean recursively all user resources in '%s'." % rootDir)

        for directory in directories:
            if self.recursiveMode:
                self.DEBUG("#> Clean user resources in '%s'." % directory)

            else:
                self.pPrint("#> Clean user resources in '%s'." % directory)

            for item in sorted(os.listdir(directory)):
                if item[0:1] != '.':
                    fileInfo = QtCore.QFileInfo(addBS(directory) + item)
                    if (fileInfo.isSymLink() or fileInfo.exists()):
                        resource = nsResource(kdecore.KUrl(fileInfo.absoluteFilePath()))
                        resource.remove()
                        if self.recursiveMode:
                            self.DEBUG("#> Resources in '%s' has been cleaned." % item)

                        else:
                            self.pPrint("#> Resources in '%s' has been cleaned." % item, 'stdout')

        self.pPrint("#> Clean directory process finished.")

        return True


    def commandCleanFile(self, url = None):
        if url == None:
            url = self.url

        if url == None or url == '':
            return False

        fileInfo = QtCore.QFileInfo(url)
        # OJO: Se pueden querer borrar datos de un archivo o directorio borrados.
        #if not (fileInfo.isSymLink() or fileInfo.exists()):
        #    return False

        resource = nsResource(kdecore.KUrl(fileInfo.absoluteFilePath()))
        resource.remove()
        self.pPrint("#> Resources in '%s' has been cleaned." % url)

        return True


    def commandEditResources(self, url = None):
        if url == None:
            url = self.url

        rootDir = self.getRootDir(url)
        if rootDir == None:
            return False

        resourceFile = addBS(rootDir) + self.localResourcesFileName

        if os.path.exists(resourceFile):
            cmd = "kioclient exec '%s'" % resourceFile
            os.system(toUtf8(cmd))
        else:
            self.pPrint("ERROR: local resource file '%s' does not exists." % resourceFile, 'error')

        return True


    def commandExport(self):
        if not self.enabled:
            self.DEBUG('Nepomuk is not running!')
            return False

        #resourceTypes = [NEO('nco#Contact'), NEO('nco#Gender'), NEO('nao#Tag'), NEO('nfo#FileDataObject'), NEO('nmm#MusicAlbum'), NEO('nco#OrganizationContact'), NEO('nco#PersonContact'), NEO('nfo#Website'), NEO('pimo#Country')]
        #resourceTypes = [NEO('nmm#MusicAlbum')]
        #resourceTypes = ['http://www.strigi.org/fields#indexGraphFor']
        #for a in `nepomukcmd --foo query "select distinct ?g where { \
        #?g <http://www.strigi.org/fields#indexGraphFor> ?r . }"`;
        #do nepomukcmd rmgraph "$a"; done
        resourceTypes = self.ontologies

        self.pPrint("#> Exporting Nepomuk user resources to '%s'..." % self.resourcesFileName)

        text = "#\n" \
                "# %(1)s export/import resource file\n" \
                "#\n" \
                "# Format:\n" \
                "#    cvs file using ',' as delimiter and '|' as quotechar\n" \
                "#" \
                % {'1': PROGRAM_BASENAME}

        if self.resourcesFileName != '--':
            outText = text + '\n'

        else:
            print text

        self.DEBUG(text)

        for resourceType in resourceTypes:

            self.pPrint("#> Looking for '%s' resources" % os.path.basename(resourceType), 'stdout')
            count, data = self.allResourcesOfType(resourceType)

            if count < 0:
                if self.resourcesFileName != '--':
                    outText += '\n'

                else:
                    print ''

                self.DEBUG('')

            elif count == 0:
                pass

            else:
                self.pPrint("#> Exporting %(count)s resources of type '%(resourceType)s'" % {'count': count, 'resourceType': os.path.basename(resourceType)}, 'stdout')
                for resource in data:
                    text = resource.toText()
                    if text != '':
                        if self.resourcesFileName != '--':
                            outText += text + '\n'

                        else:
                            print text

                        self.pPrint("#> Resource '%(id)s' of type '%(type)s' has been exported." % {'id': resource.ID, 'type': os.path.basename(toUnicode(resource.type()))}, 'stdout')

        if self.resourcesFileName != '--':
            try:
                if self.backupFiles and os.path.isfile(self.resourcesFileName):
                    os.rename(self.resourcesFileName, self.resourcesFileName + self.backupExtension)

                f = codecs.open(self.resourcesFileName, encoding = gSysEncoding, mode = 'w+')
                try:
                    f.write(toUnicode(outText))

                except:
                    self.pPrint("ERROR: resources file '%s' can't be written." % self.resourcesFileName, 'error')

                finally:
                    f.close()

                self.pPrint("#> Export process is complete.")

            except:
                self.pPrint("ERROR: resources file '%s' can't be created." % self.resourcesFileName, 'error')

        else:
            self.pPrint("#> Export process is complete.")

        return True


    def commandHelp(self):
        if self.serviceMenuMode:
            self.pPrint("<b>%(1)s</b> <b>%(2)s</b> (%(3)s) - by <b>%(4)s</b> (<em>%(5)s</em>)<br \><br \>" \
                "<b>%(7)s</b> <em>service menu</em> is a collection of tools to manage some <em>Nepomuk user data</em> (comments, ratings and tags)<br /><br />" \
                "Tip: try '%(1)s' command in console for more options and information</em><br />" \
                "Tip: use local resources to share information between users and systems (usb disks)<br />" \
                "Tip: you can use find to add tags massively: find . -name *.avi -exec %(1)s --tag 'pretty tag' --url {} \;<br />" \
                "Tip: use export/import options manually or with cron to backup all your user data<br /><br />" \
                "Report bugs, ideas and suggestions to '%(5)s'<br />" \
                "My first python program so comments to improve my code are welcome :-)<br /><br />" \
                "<b>Warning!</b> Be cautious, proper error handling is not implemented yet so if<br />" \
                "something goes wrong try the same in console<br /><br />" \
                "<em>available options</em>:<ul>" \
                "<li>Save resources to a local file:<br />" \
                "    Save local user resources to associated '%(8)s' (recursive mode is available).<br /></li>" \
                "<li>Load resources from a local file:<br />" \
                "    Load local user resources from associated '%(8)s' (recursive mode is available).<br /></li>" \
                "<li>Clean resources in a directory:<br />" \
                "    Delete all resources in selected directory (recursive mode is available).<br />Remarks: local resource file is not removed.<br /></li>" \
                "<li>Clean file resources:<br />" \
                "    Delete all resources in selected file.<br />Remarks: data in local resource file is not removed.<br /></li>" \
                "<li>Edit associated local resources file:<br />" \
                "    Associated local resource file '%(8)s' is edited with default text editor.<br /></li>" \
                "<li>Remove associated local resources file:<br />" \
                "    Associated local resource file '%(8)s' is removed.<br /></li>" \
                "<li>Export all user data:<br />" \
                "    Exports all user data to file '%(6)s'<br />" \
                "    Warning!, this operation may be slow.<br /></li>" \
                "<li>Import all user data:<br />" \
                "    Imports all user data from file '%(6)s''<br />" \
                "    Warning!, this operation may be very slow.<br /></li>" \
                "<li>Help:<br \>" \
                "    Shows this help.</li>" \
                "</ul>" \
                "<em>future options</em>:<ul>" \
                "<li>safe delete and move</li><li>restart soprano database</li><li>monitoring mode</li><li>automatic usb disk handling</li><li>etc...</li>" \
                "</ul>" \
                % {'1': PROGRAM_NAME, '2': PROGRAM_VERSION_VERSION, '3': PROGRAM_VERSION_DATE, '4': PROGRAM_AUTHOR_NAME, '5': PROGRAM_AUTHOR_EMAIL, '6': self.resourcesFileName, '7': PROGRAM_BASENAME, '8': self.localResourcesFileName}, 'msgbox')

        else:
            self.pPrint("%(1)s %(2)s (%(3)s) - by %(4)s (%(5)s)\n\n" \
                "usage:\n" \
                "  %(1)s command [options]\n" \
                "\n" \
                "commands:\n" \
                "  addtags       - one or more tags specified in --tag are added to --url\n" \
                #"  cleanall      - cleans all user resources\n" \
                "  cleandir      - cleans all file resources in --url directory\n" \
                "  cleanfile     - cleans all resources in --url file\n" \
                #"  delete        - delete an item and associated resources\n" \
                "  editresources - edit local resources file (%(7)s) associated to --url\n" \
                "  export        - exports all resources to a file (could be slow)\n" \
                "  import        - import all resources from a file (could be very slow)\n" \
                "  install       - configure service menu\n" \
                "  help          - shows this help\n" \
                "  listresources - list file or directory resources (only tags, ratings and comments)\n" \
                #"  loadall       - load all file resources from associated local resource files (could be very slow)\n" \
                "  loaddir       - load from local resources file all file resources in --url directory\n" \
                #"  loadfile       - load file resources from associated local resource file\n" \
                #"  move          - move an item mantaining user resources\n" \
                #"  restartdb     - restart nepomuk with an empty db (warning, all data is lost)\n" \
                #"  saveall       - save all file resources to associated local resource files (could be very slow)\n" \
                "  removetags    - one or more tags specified in --tag are removed from --url\n" \
                "  savedir       - save to local resources file all file resources in --url directory\n" \
                #"  savefile       - save selected file resources to associated local resources file\n" \
                "  remove        - removes local resources file associated to --url \n" \
                #"  removeall     - removes all system file resources\n" \
                "  uninstall     - removes service menu\n" \
                "\n" \
                "options:\n" \
                "  -h | --human-readable - local resources file is created in a format easy to understand\n" \
                "  -n | --no-backup      - resource file is not backed up\n" \
                "  -o | --ontology       - ontology filter (default = nao#Tag, nfo#FileDataObject)\n" \
                "  -R | --recursive      - recursive mode (only available with certain commands)\n" \
                "  -r | --resourcefile   - resource file (current = \"%(6)s)\"\n" \
                "  -s | --servicemenu    - service menu mode (instead stdout KDE notifications is used)\n" \
                "  -t | --tag <tag>      - tag identifier required by 'settag' and 'removetag' commands\n" \
                "  -u | --url <url>      - working url (file, directory, symlink or nepomuk uri)\n" \
                "\n" \
                "examples:\n" \
                "  neposidekick.py loaddir --recursive --url ~/Documents\n" \
                "  neposidekick.py savedir --human-readable --recursive --url ~/Documents\n" \
                "  neposidekick.py addtags --tag 'Tag One' --tag 'Tag Two' --url ~/Documents\n" \
                % {'1': PROGRAM_NAME, '2': PROGRAM_VERSION_VERSION, '3': PROGRAM_VERSION_DATE, '4': PROGRAM_AUTHOR_NAME, '5': PROGRAM_AUTHOR_EMAIL, '6': self.resourcesFileName, '7': self.localResourcesFileName}, 'stdout')

        return True


    def commandImport(self):
        if not self.enabled:
            self.DEBUG('Nepomuk is not running!')
            return False

        self.pPrint("#> Importing Nepomuk user resources from '%s'..." % self.resourcesFileName)
        fileData = csv.reader(open(self.resourcesFileName, 'rb'), delimiter=',', quotechar='|')
        i = 0
        for row in fileData:
            if row == [] or row[0].strip()[0] == "#":
                continue

            i += 1
            try:
                resource = nsResource(self, row)
                if resource.ID != None:
                    self.pPrint("#> Resource '%(id)s' of type '%(type)s' has been imported." % {'id': resource.ID, 'type': os.path.basename(toUnicode(resource.type()))}, 'stdout')

            except:
                self.pPrint("#> Failed to import row %s." % i, 'error')

        self.pPrint("#> %s rows processed." % i, 'stdout')
        self.pPrint("#> Import process is complete.")

        return True


    def commandInstall(self):
        serviceMenuFile = os.popen('kde4-config --path services').read().strip()
        serviceMenuFile = serviceMenuFile[0:serviceMenuFile.find(':')] + '/ServiceMenus/' + PROGRAM_BASENAME + '.desktop'
        text = '[Desktop Action help]\n' \
                'Exec=%(1)s help --servicemenu --url "%(2)s"\n' \
                'Icon=help-about\n' \
                'Name=Help\n' \
                '\n' \
                '[Desktop Action cleandir]\n' \
                'Exec=%(1)s cleandir --servicemenu --url "%(2)s"\n' \
                'Icon=trash-empty\n' \
                'Name=Clean resources in a directory\n' \
                '\n' \
                '[Desktop Action cleandirr]\n' \
                'Exec=%(1)s cleandir --servicemenu --recursive --url "%(2)s"\n' \
                'Icon=trash-empty\n' \
                'Name=Clean resources in a directory (recursive)\n' \
                '\n' \
                '[Desktop Action cleanfile]\n' \
                'Exec=%(1)s cleanfile --servicemenu --url "%(2)s"\n' \
                'Icon=trash-empty\n' \
                'Name=Clean file resources\n' \
                '\n' \
                '[Desktop Action export]\n' \
                'Exec=%(1)s export --servicemenu --url "%(2)s"\n' \
                'Icon=document-export\n' \
                'Name=Export all user data\n' \
                '\n' \
                '[Desktop Action import]\n' \
                'Exec=%(1)s import --servicemenu --url "%(2)s"\n' \
                'Icon=document-import\n' \
                'Name=Import all user data\n' \
                '\n' \
                '[Desktop Action savedir]\n' \
                'Exec=%(1)s savedir --servicemenu --human-readable --url "%(2)s"\n' \
                'Icon=document-save\n' \
                'Name=Save resources to a local file\n' \
                '\n' \
                '[Desktop Action savedirr]\n' \
                'Exec=%(1)s savedir --servicemenu --recursive --url "%(2)s"\n' \
                'Icon=document-save\n' \
                'Name=Save resources to a local file (recursive)\n' \
                '\n' \
                '[Desktop Action loaddir]\n' \
                'Exec=%(1)s loaddir --servicemenu --url "%(2)s"\n' \
                'Icon=document-open\n' \
                'Name=Load resources from a local file\n' \
                '\n' \
                '[Desktop Action loaddirr]\n' \
                'Exec=%(1)s loaddir --servicemenu --recursive --url "%(2)s"\n' \
                'Icon=document-open\n' \
                'Name=Load resources from a local file (recursive)\n' \
                '\n' \
                '[Desktop Action editresources]\n' \
                'Exec=%(1)s editresources --servicemenu --url "%(2)s"\n' \
                'Icon=accessories-text-editor\n' \
                'Name=Edit local resources file\n' \
                '\n' \
                '[Desktop Action remove]\n' \
                'Exec=%(1)s remove --servicemenu --url "%(2)s"\n' \
                'Icon=archive-remove\n' \
                'Name=Remove local resources file\n' \
                '\n' \
                '[Desktop Entry]\n' \
                'Actions=loaddir;loaddirr;_SEPARATOR_;savedir;savedirr;_SEPARATOR_;cleandir;cleandirr;cleanfile;_SEPARATOR_;editresources;remove;_SEPARATOR_;export;import;_SEPARATOR_;help;\n' \
                'Encoding=UTF-8\n' \
                'Icon=nepomuk\n' \
                'ServiceTypes=KonqPopupMenu/Plugin,all/allfiles,inode/directory\n' \
                'Type=Service\n' \
                'X-KDE-Priority=TopLevel\n' \
                'X-KDE-Submenu=Neposidekick\n' \
                % {'1': PROGRAM_NAME, '2': '%u'}

        try:
            f = codecs.open(serviceMenuFile, encoding = gSysEncoding, mode = 'w+')
            try:
                f.write(toUnicode(text))

            except:
                self.pPrint("ERROR: service menú file '%s' can't be written." % serviceMenuFile, 'error')

            finally:
                f.close()

            cmd = 'kbuildsycoca4 > /dev/null 2> /dev/null'
            os.system(toUtf8(cmd))
            self.pPrint("%(1)s service menu installed\n\nWarning: please note that if '%(2)s' is not in a directory in $PATH the service menu will not work" % {'1': PROGRAM_BASENAME, '2': PROGRAM_NAME})

        except:
            self.pPrint("ERROR: service menú file '%s' can't be created." % serviceMenuFile, 'error')

        return True


    def commandListResources(self, url = None):
        if url == None:
            url = self.url

        rootDir = self.getRootDir(url)
        if rootDir == None:
            return False

        if self.recursiveMode:
            directoriesCount, directories = findRecurseDirectories(rootDir)

        else:
            directoriesCount = 1
            directories = [addBS(rootDir)]

        for directory in directories:

            for item in sorted(os.listdir(directory)):
                if item[0:1] != '.':
                    fileInfo = QtCore.QFileInfo(addBS(directory) + item)
                    if (fileInfo.isSymLink() or fileInfo.exists()):
                        resource = nsResource(kdecore.KUrl(fileInfo.absoluteFilePath()))
                        text = resource.toText('fl')

                        if text == '':
                            resource.remove()

                        else:
                            self.pPrint(text, 'stdout')

        return True


    #def commandLoadAll(self):
        #self.commandHelp()

        #return True


    def commandLoadDir(self, url = None):
        if url == None:
            url = self.url

        rootDir = self.getRootDir(url)
        if rootDir == None:
            return False

        if self.recursiveMode:
            directoriesCount, directories = findRecurseDirectories(rootDir)

        else:
            directoriesCount = 1
            directories = [addBS(rootDir)]

        if self.recursiveMode:
            self.pPrint("#> Loading recursively all user resources from '%s'." % rootDir)

        for directory in directories:
            if self.resourcesFileName == DEFAULT_RESOURCE_FILE:
                resourceFile = directory + self.localResourcesFileName

            else:
                if self.resourcesFileName.find('/') >= 0:
                    resourceFile = self.resourcesFileName

                else:
                    resourceFile = directory + self.resourcesFileName

            try:
                fileData = csv.reader(open(resourceFile, 'rb'), delimiter=',', quotechar='|')

            except:
                self.DEBUG("Directory %s skipped." % directory)
                fileData = None

            if fileData != None:
                if self.recursiveMode:
                    self.DEBUG("#> Loading user resources from '%s'." % resourceFile)

                else:
                    self.pPrint("#> Loading user resources from '%s'." % resourceFile, 'stdout')

                for row in fileData:
                    if row == [] or row[0].strip()[0] == "#":
                        continue

                    resource = nsResource(self, row, directory)
                    try:
                        if resource.ID == None:
                            # No tenía datos o algo falló, lo quitamos por seguridad.
                            resource.remove()

                        else:
                            if self.recursiveMode:
                                self.DEBUG("#> Resources for file '%s' has been loaded." % resource.ID)

                            else:
                                self.pPrint("#> Resources for file '%s' has been loaded." % resource.ID, 'stdout')

                    except:
                        pass


        self.pPrint("#> Load directory process finished.")

        return True


    #def commandLoadFile(self, url = None):
        #self.commandHelp()

        #return True


    #def commandMonitor(self):
        #self.commandHelp()

        #return True


    def commandRemove(self, url = None):
        if url == None:
            url = self.url

        rootDir = self.getRootDir(url)
        if rootDir == None:
            return False

        if self.recursiveMode:
            resourcesCount, resources = findResourceFiles(rootDir)

        else:
            resourcesCount = 1
            resources = [addBS(rootDir) + self.localResourcesFileName]

        for resourceFile in resources:
            filesRemoved = False
            if os.path.exists(resourceFile):
                try:
                    os.remove(resourceFile)
                    filesRemoved = True

                except:
                    pass

            if os.path.exists(resourceFile + self.backupExtension):
                try:
                    os.remove(resourceFile + self.backupExtension)
                    filesRemoved = True

                except:
                    pass

            if filesRemoved:
                self.pPrint("#> Local resource file '%s' deleted." % resourceFile)

        return True


    def commandRemoveTags(self, url = None, tags = []):
        if url == None:
            url = self.url

        if url == None or url == '':
            return False

        fileInfo = QtCore.QFileInfo(url)
        if not (fileInfo.isSymLink() or fileInfo.exists()):
            return False

        if tags == []:
            tags = self.tags

        if tags == []:
            return True

        try:
            oResource = nsResource(kdecore.KUrl(fileInfo.absoluteFilePath()))
            strTags = ''
            displayName = 'tag'
            for tag in tags:
                oTag = Nepomuk.Tag(tag)
                oTag.setLabel(tag)
                oResource.removeProperty(QtCore.QUrl(NEO('nao#hasTag')), Nepomuk.Variant(oTag.resourceUri()))
                if strTags != '':
                    if tag == tags[len(tags) - 1]:
                        strTags += ' and '
                        displayName += 's'

                    else:
                        strTags += ', '

                strTags += "'%s'" % tag

            self.pPrint("#> %(tags)s %(displayname)s removed from '%(url)s'." % {'tags': strTags, 'displayname': displayName, 'url': url})

        except:
            self.pPrint("#> Failed to remove tags to '%s'." %  url)

        return True


    #def commandSaveAll(self):
        #self.commandHelp()

        #return True


    def commandSaveDir(self, url = None):
        if url == None:
            url = self.url

        rootDir = self.getRootDir(url)
        if rootDir == None:
            return False

        if self.recursiveMode:
            directoriesCount, directories = findRecurseDirectories(rootDir)

        else:
            directoriesCount = 1
            directories = [addBS(rootDir)]

        if self.recursiveMode:
            self.pPrint("#> Saving recursively all user resources to '%s'." % rootDir)

        for directory in directories:
            if self.resourcesFileName == DEFAULT_RESOURCE_FILE:
                resourceFile = directory + self.localResourcesFileName

            else:
                if self.resourcesFileName.find('/') >= 0:
                    resourceFile = self.resourcesFileName

                else:
                    resourceFile = directory + self.resourcesFileName

            if self.outputFormat == 'human':
                outText = "#\n" \
                        "# %(1)s local resource file\n" \
                        "#\n" \
                        "# Format:\n" \
                        "#    cvs file using ',' as delimiter and '|' as quotechar\n" \
                        "#\n" \
                        "# Columns:\n" \
                        "#    reserved: nhr\n" \
                        "#        file: file name without path\n" \
                        "#     comment: some useful text\n" \
                        "#      rating: 0..10 or an empty value\n" \
                        "#         tag: one tag per column\n" \
                        "#\n" \
                        "# Examples:\n" \
                        "#    nhr,|example1.txt|,|rating 1 and 3 tags|,|1|,|tag 1|,|tag 2|,|tag 3|\n" \
                        "#    nhr,|example2.txt|,|a comment with\n" \
                        "#    an intro|,|5|,|tag 1|\n" \
                        "#    nhr,|example3.txt|,|no tags and no rating|\n" \
                        "#\n" \
                        % {'1': PROGRAM_BASENAME}

            elif self.outputFormat == 'list':
                pass

            else:
                outText = "#\n" \
                        "# %(1)s export/import resource file\n" \
                        "#\n" \
                        "# Format:\n" \
                        "#    cvs file using ',' as delimiter and '|' as quotechar\n" \
                        "#\n" \
                        % {'1': PROGRAM_BASENAME}

            if self.recursiveMode:
                self.DEBUG("#> Saving user resources to '%s'." % resourceFile)

            else:
                self.pPrint("#> Saving user resources to '%s'." % resourceFile, 'stdout')

            for item in sorted(os.listdir(directory)):
                if item[0:1] != '.':
                    fileInfo = QtCore.QFileInfo(addBS(directory) + item)
                    if (fileInfo.isSymLink() or fileInfo.exists()):
                        resource = nsResource(kdecore.KUrl(fileInfo.absoluteFilePath()))
                        if self.outputFormat == 'human':
                            text = resource.toText('fh')

                        elif self.outputFormat == 'list':
                            pass

                        else:
                            text = resource.toText('fs')

                        if text == '':
                            resource.remove()

                        else:
                            if self.resourcesFileName == '--':
                                print toUtf8(text)

                            else:
                                outText += text + '\n'

                        if self.recursiveMode:
                            self.DEBUG("Resources in file '%s' has been exported." % item)

                        else:
                            self.pPrint("Resources in file '%s' has been exported." % item, 'stdout')

            if self.resourcesFileName != '--':
                try:
                    if self.backupFiles and os.path.isfile(resourceFile):
                        os.rename(resourceFile, resourceFile + self.backupExtension)

                    f = codecs.open(resourceFile, encoding = gSysEncoding, mode = 'w+')
                    try:
                        f.write(toUnicode(outText))

                    except:
                        self.pPrint("ERROR: resources file '%s' can't be written." % resourceFile, 'error')

                    finally:
                        f.close()

                    if not self.recursiveMode:
                        self.pPrint("#> Save directory process finished.")

                except:
                    self.pPrint("ERROR: resources file '%s' can't be created." % resourceFile, 'error')

            else:
                if not self.recursiveMode:
                    self.pPrint("#> Save directory process finished.")

        if self.recursiveMode:
            self.pPrint("#> Save directory process finished.")

        return True


    #def commandSaveFile(self, url = None):
        #self.commandHelp()

        #return True


    #def commandSetComment(self, url = None):
        #self.commandHelp()

        #return True


    def commandUninstall(self):
        serviceMenuFile = os.popen('kde4-config --path services').read().strip()
        serviceMenuFile = serviceMenuFile[0:serviceMenuFile.find(':')] + '/ServiceMenus/' + PROGRAM_BASENAME + '.desktop'

        try:
            os.remove(serviceMenuFile)
            cmd = 'kbuildsycoca4 > /dev/null 2> /dev/null'
            os.system(toUtf8(cmd))
            self.pPrint('%s service menu uninstalled.' % PROGRAM_NAME)

        except:
            self.pPrint("ERROR: service menú file '%s' can't be deleted." % serviceMenuFile)

        return True


    def runCommand(self, command = None):
        if not self.enabled:
            self.DEBUG('Nepomuk is not running!')
            return False

        if command == None or command == '':
            command = self.command
            if command == None or command == '':
                command = 'help'

        commandName = command.capitalize()
        if not (commandName == 'Install' or commandName == 'Uninstall'):
            commandName = commandName.replace('all', 'All')
            commandName = commandName.replace('dir', 'Dir')
            commandName = commandName.replace('file', 'File')
            commandName = commandName.replace('resource', 'Resource')
            commandName = commandName.replace('tag', 'Tag')

        #exec 'self.command%s()' % commandName
        try:
            func = getattr(self, 'command%s' % commandName)
        except:
            self.pPrint("ERROR: unknown command '%s'." % command)
            return False

        func()
        self.lastCommand = command

        return True


    def parseParameters(self, parameters):
        parameter = ''
        ontologiesCleared = False

        for i in range(1, len(parameters)):
            item = parameters[i].strip()

            if item == '--help':
                self.command = 'help'
                return True

            elif item == '-h' or item == '--human-readable':
                self.outputFormat = 'human'
                continue

            elif item == '-n' or item == '--no-backup':
                self.backupFiles = False
                return True

            elif item == '-o' or item == '--ontology':
                parameter = 'o'

            elif item == '-r' or item == '--resourcefile':
                parameter = 'r'

            elif item == '-R' or item == '--recursive':
                self.recursiveMode = True

            elif item == '-s' or item == '--servicemenu':
                self.serviceMenuMode = True

            elif item == '-t' or item == '--tag':
                parameter = 't'

            elif item == '-u' or item == '--url':
                parameter = 'u'

            elif item == '--debug':
                parameter = 'debug'

            else:
                if parameter == 'o':
                    if item == 'default':
                        self.ontologies = self.defaultOntologies

                    else:
                        if not ontologiesCleared:
                            self.ontologies = []

                        if not item in self.ontologies:
                            if item[0:7] == 'http://':
                                self.ontologies += [item]

                            else:
                                self.ontologies += [NEO(item)]

                elif parameter == 'r':
                    self.resourcesFileName = item

                elif parameter == 't':
                    self.tags += [toUnicode(item)]

                elif parameter == 'u':
                    if item[0:7] == 'file://':
                        item = item[7:]
                    self.url = toUnicode(item)

                elif parameter == 'debug':
                    if item.strip().lower() == 'on':
                        self.DEBUG_ENABLED = True

                    elif item.strip().lower() == 'off':
                        self.DEBUG_ENABLED = False

                    else:
                        pass

                else:
                    self.command = item

                parameter = ''

        return True

# end  nsprogram.py
# begin_2 neposidekick.py
def main():
    nsp = nsProgram(sys.argv)


if __name__ == '__main__':
    main()
# end_2 neposidekick.py
